/*
 * Copyright (c) 2021 NetEase, Inc.  All rights reserved.
 * Use of this source code is governed by a MIT license that can be
 * found in the LICENSE file.
 */

package com.netease.nimflutter

import android.os.Handler;
import android.os.Looper;
import io.flutter.plugin.common.MethodChannel.Result

class SafeResult(private val unsafeResult: Result) : Result {

    private var handler: Handler = Handler(Looper.getMainLooper())

    override fun success(result: Any?) {
        runOnMainThread { unsafeResult.success(result) }
    }

    override fun error(errorCode: String?, errorMessage: String?, errorDetails: Any?) {
        runOnMainThread { unsafeResult.error(errorCode, errorMessage, errorDetails) }
    }

    override fun notImplemented() {
        runOnMainThread(unsafeResult::notImplemented)
    }

    private fun runOnMainThread(runnable: () -> Unit) {
        if (Looper.getMainLooper() == Looper.myLooper()) {
            runnable()
        } else {
            handler.post(runnable)
        }
    }
}