// Copyright (c) 2021 NetEase, Inc.  All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'dart:convert';

import 'package:nim_core/nim_core.dart';
import 'package:yunxin_integration_test/yunxin_integration_test.dart';

class HandleEventSubscribeCase extends HandleBaseCase {
  HandleEventSubscribeCase();

  @override
  Future<ResultBean?> handle(event) async {
    super.handle(event);
    switch (methodName) {
      case "registerEventSubscribe":
        var inputParams = jsonDecode(params![0]) as Map<String, dynamic>;
        var publish = inputParams['publishers'] as List<dynamic>;
        List<String> publisherList = publish.map((e) => e.toString()).toList();
        EventSubscribeRequest request = EventSubscribeRequest(
            eventType: inputParams['eventType'],
            expiry: inputParams['expiry'],
            publishers: publisherList);
        final result = await NimCore.instance.eventSubscribeService
            .registerEventSubscribe(request);
        print(
            '=========>>eventSubscribeService:${result.code}：${result.errorDetails}');
        handleCaseResult =
            ResultBean(code: result.code, data: result.toString(), message: '');
        break;

      case "batchUnSubscribeEvent":
        var inputParams = jsonDecode(params![0]) as Map<String, dynamic>;
        var publish = inputParams['publishers'] as List<dynamic>;
        List<String> publisherList = publish.map((e) => e.toString()).toList();
        EventSubscribeRequest request = EventSubscribeRequest(
            eventType: inputParams['eventType'],
            expiry: inputParams['expiry'],
            publishers: publisherList);
        final result = await NimCore.instance.eventSubscribeService
            .batchUnSubscribeEvent(request);
        print(
            '=========>>eventSubscribeService:${result.code}：${result.errorDetails}');
        handleCaseResult =
            ResultBean(code: result.code, data: result.toString(), message: '');
        break;

      case "unregisterEventSubscribe":
        var inputParams = jsonDecode(params![0]) as Map<String, dynamic>;
        var publish = inputParams['publishers'] as List<dynamic>;
        List<String> publisherList = publish.map((e) => e.toString()).toList();
        EventSubscribeRequest request = EventSubscribeRequest(
            eventType: inputParams['eventType'],
            expiry: inputParams['expiry'],
            publishers: publisherList);
        final result = await NimCore.instance.eventSubscribeService
            .unregisterEventSubscribe(request);
        print(
            '=========>>eventSubscribeService:${result.code}：${result.errorDetails}');
        handleCaseResult =
            ResultBean(code: result.code, data: result.toString(), message: '');
        break;

      case "publishEvent":
        var inputParams = jsonDecode(params![0]) as Map<String, dynamic>;
        Event event = Event.fromMap(inputParams);
        final result =
            await NimCore.instance.eventSubscribeService.publishEvent(event);
        print(
            '=========>>eventSubscribeService:${result.code}：${result.errorDetails}');
        handleCaseResult =
            ResultBean(code: result.code, data: result.toString(), message: '');
        break;

      case "querySubscribeEvent":
        var inputParams = jsonDecode(params![0]) as Map<String, dynamic>;
        var publish = inputParams['publishers'] as List<dynamic>;
        List<String> publisherList = publish.map((e) => e.toString()).toList();
        EventSubscribeRequest request = EventSubscribeRequest(
            eventType: inputParams['eventType'],
            expiry: inputParams['expiry'],
            publishers: publisherList);
        final result = await NimCore.instance.eventSubscribeService
            .querySubscribeEvent(request);
        print(
            '=========>>eventSubscribeService:${result.code}：${result.errorDetails}');
        handleCaseResult =
            ResultBean(code: result.code, data: result.toString(), message: '');
        break;
    }

    return handleCaseResult;
  }
}
