// Copyright (c) 2021 NetEase, Inc.  All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'dart:convert';
import 'dart:io';

import 'package:nim_core/nim_core.dart';
import 'package:yunxin_integration_test/yunxin_integration_test.dart';

import 'method_name_value.dart';

/// 配置case用例地址： https://g.hz.netease.com/yunxin-app/kit_automation_test/-/tree/release/integration_case
/// case模板，模板代码的class需要在 [nim_core_test.dart] 中注册。
/// 执行结果通过返回[handleCaseResult] ,做期望值匹配
/// 执行方式,请不要在本地执行：在packages/nim_core/nim_core/example目录下
/// flutter drive --driver=test_driver/integration_test.dart --target=integration_test/main_test.dart  --keep-app-running
class HandleInitializeCase extends HandleBaseCase {
  HandleInitializeCase();

  @override
  Future<ResultBean?> handle(event) async {
    super.handle(event);
    switch (methodName) {
      case initialize:
        Map map = jsonDecode(params![0]);
        Map optionsMap = map.entries.first.value;
        final result = await NimCore.instance.initialize(Platform.isAndroid
            ? NIMAndroidSDKOptions.fromMap(optionsMap)
            : NIMIOSSDKOptions.fromMap(optionsMap));
        print('=========>>initialize: ${result.code}');
        handleCaseResult = ResultBean(
          code: result.code,
          data: result.toMap(),
          message: result.errorDetails,
        );
        break;
    }

    return handleCaseResult;
  }
}
