// Copyright (c) 2021 NetEase, Inc.  All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'dart:convert';
import 'dart:io';

import 'package:nim_core/nim_core.dart';
import 'package:yunxin_integration_test/yunxin_integration_test.dart';

class HandleSystemMessageCase extends HandleBaseCase {
  HandleSystemMessageCase();

  @override
  Future<ResultBean?> handle(event) async {
    super.handle(event);
    switch (methodName) {
      case "querySystemMessages":
        if (Platform.isAndroid) {
          var inputParams = jsonDecode(params![0]) as Map<String, dynamic>;
          final result = await NimCore.instance.systemMessageService
              .querySystemMessages(inputParams['limit'] as int);
          print(
              '=========>>systemMessageService:${result.code}：${result.errorDetails}');
          handleCaseResult =
              ResultBean(code: result.code, data: result.data, message: '');
        } else {
          handleCaseResult = ResultBean.success(message: "ios default success");
        }
        break;

      case "querySystemMessageByType":
        if (Platform.isAndroid) {
          var inputParams = jsonDecode(params![0]) as Map<String, dynamic>;
          var paramList = inputParams['systemMessageTypeList'] as List<dynamic>;
          List<SystemMessageType> systemMessageTypeList = paramList
              .map((e) => SystemMessageTypeConverter().fromValue(e as String))
              .toList();
          final result = await NimCore.instance.systemMessageService
              .querySystemMessageByType(systemMessageTypeList,
                  inputParams['offset'] as int, inputParams['limit'] as int);
          print(
              '=========>>systemMessageService:${result.code}：${result.errorDetails}');
          handleCaseResult = ResultBean(
              code: result.code, data: result.toString(), message: '');
        } else {
          handleCaseResult = ResultBean.success(message: "ios default success");
        }
        break;

      case "querySystemMessageUnread":
        if (Platform.isAndroid) {
          // var inputParams = jsonDecode(params![0]) as Map<String, dynamic>;
          final result = await NimCore.instance.systemMessageService
              .querySystemMessageUnread();
          print(
              '=========>>systemMessageService:${result.code}：${result.errorDetails}');
          handleCaseResult = ResultBean(
              code: result.code, data: result.toString(), message: '');
        } else {
          handleCaseResult = ResultBean.success(message: "ios default success");
        }
        break;

      case "querySystemMessageUnreadCount":
        if (Platform.isAndroid) {
          // var inputParams = jsonDecode(params![0]) as Map<String, dynamic>;
          final result = await NimCore.instance.systemMessageService
              .querySystemMessageUnreadCount();
          print(
              '=========>>systemMessageService:${result.code}：${result.errorDetails}');
          handleCaseResult = ResultBean(
              code: result.code, data: result.toString(), message: '');
        } else {
          handleCaseResult = ResultBean.success(message: "ios default success");
        }
        break;

      case "querySystemMessageUnreadCountByType":
        if (Platform.isAndroid) {
          var inputParams = jsonDecode(params![0]) as Map<String, dynamic>;
          var paramList = inputParams['systemMessageTypeList'] as List<dynamic>;
          List<SystemMessageType> systemMessageTypeList = paramList
              .map((e) => SystemMessageTypeConverter().fromValue(e as String))
              .toList();
          final result = await NimCore.instance.systemMessageService
              .querySystemMessageUnreadCountByType(systemMessageTypeList);
          print(
              '=========>>systemMessageService:${result.code}：${result.errorDetails}');
          handleCaseResult = ResultBean(
              code: result.code, data: result.toString(), message: '');
        } else {
          handleCaseResult = ResultBean.success(message: "ios default success");
        }
        break;

      case "resetSystemMessageUnreadCount":
        if (Platform.isAndroid) {
          // var inputParams = jsonDecode(params![0]) as Map<String, dynamic>;
          final result = await NimCore.instance.systemMessageService
              .resetSystemMessageUnreadCount();
          print(
              '=========>>systemMessageService:${result.code}：${result.errorDetails}');
          handleCaseResult = ResultBean(
              code: result.code, data: result.toString(), message: '');
        } else {
          handleCaseResult = ResultBean.success(message: "ios default success");
        }
        break;

      case "resetSystemMessageUnreadCountByType":
        if (Platform.isAndroid) {
          var inputParams = jsonDecode(params![0]) as Map<String, dynamic>;
          var paramList = inputParams['systemMessageTypeList'] as List<dynamic>;
          List<SystemMessageType> systemMessageTypeList = paramList
              .map((e) => SystemMessageTypeConverter().fromValue(e as String))
              .toList();
          final result = await NimCore.instance.systemMessageService
              .resetSystemMessageUnreadCountByType(systemMessageTypeList);
          print(
              '=========>>systemMessageService:${result.code}：${result.errorDetails}');
          handleCaseResult = ResultBean(
              code: result.code, data: result.toString(), message: '');
        } else {
          handleCaseResult = ResultBean.success(message: "ios default success");
        }
        break;

      case "setSystemMessageRead":
        if (Platform.isAndroid) {
          var inputParams = jsonDecode(params![0]) as Map<String, dynamic>;
          final result = await NimCore.instance.systemMessageService
              .setSystemMessageRead(inputParams['messageId'] as int);
          print(
              '=========>>systemMessageService:${result.code}：${result.errorDetails}');
          handleCaseResult = ResultBean(
              code: result.code, data: result.toString(), message: '');
        } else {
          handleCaseResult = ResultBean.success(message: "ios default success");
        }
        break;

      case "clearSystemMessages":
        if (Platform.isAndroid) {
          // var inputParams = jsonDecode(params![0]) as Map<String, dynamic>;
          final result =
              await NimCore.instance.systemMessageService.clearSystemMessages();
          print(
              '=========>>systemMessageService:${result.code}：${result.errorDetails}');
          handleCaseResult = ResultBean(
              code: result.code, data: result.toString(), message: '');
        } else {
          handleCaseResult = ResultBean.success(message: "ios default success");
        }
        break;

      case "clearSystemMessagesByType":
        if (Platform.isAndroid) {
          var inputParams = jsonDecode(params![0]) as Map<String, dynamic>;
          var paramList = inputParams['systemMessageTypeList'] as List<dynamic>;
          List<SystemMessageType> systemMessageTypeList = paramList
              .map((e) => SystemMessageTypeConverter().fromValue(e as String))
              .toList();
          final result = await NimCore.instance.systemMessageService
              .clearSystemMessagesByType(systemMessageTypeList);
          print(
              '=========>>systemMessageService:${result.code}：${result.errorDetails}');
          handleCaseResult = ResultBean(
              code: result.code, data: result.toString(), message: '');
        } else {
          handleCaseResult = ResultBean.success(message: "ios default success");
        }
        break;
      case "deleteSystemMessage":
        if (Platform.isAndroid) {
          var inputParams = jsonDecode(params![0]) as Map<String, dynamic>;
          final result = await NimCore.instance.systemMessageService
              .deleteSystemMessage(inputParams['messageId'] as int);
          print(
              '=========>>systemMessageService:${result.code}：${result.errorDetails}');
          handleCaseResult = ResultBean(
              code: result.code, data: result.toString(), message: '');
        } else {
          handleCaseResult = ResultBean.success(message: "ios default success");
        }
        break;

      case "setSystemMessageStatus":
        if (Platform.isAndroid) {
          var inputParams = jsonDecode(params![0]) as Map<String, dynamic>;
          final result = await NimCore.instance.systemMessageService
              .setSystemMessageStatus(
                  inputParams['messageId'] as int,
                  SystemMessageStatusConverter()
                      .fromValue(inputParams['systemMessageStatus'] as String));
          print(
              '=========>>systemMessageService:${result.code}：${result.errorDetails}');
          handleCaseResult = ResultBean(
              code: result.code, data: result.toString(), message: '');
        } else {
          handleCaseResult = ResultBean.success(message: "ios default success");
        }
        break;

      case "sendCustomNotification":
        if (Platform.isAndroid) {
          var inputParams = jsonDecode(params![0]) as Map<String, dynamic>;
          final result = await NimCore.instance.systemMessageService
              .sendCustomNotification(CustomNotification.fromMap(inputParams));
          print(
              '=========>>systemMessageService:${result.code}：${result.errorDetails}');
          handleCaseResult = ResultBean(
              code: result.code, data: result.toString(), message: '');
        } else {
          handleCaseResult = ResultBean.success(message: "ios default success");
        }
        break;
    }

    return handleCaseResult;
  }
}
