/*
 * Copyright (c) 2021 NetEase, Inc.  All rights reserved.
 * Use of this source code is governed by a MIT license that can be
 * found in the LICENSE file.
 */

import Foundation

let kFLTNimCoreService = "serviceName"

class NimCore {
    
    private var services = [String : FLTService]()
    
    private var safeMethodChannel: SafeMethodChannel?
    
    private var controller: UIViewController?
    
    init(_ channel: SafeMethodChannel?, _ controller: UIViewController?) {
        safeMethodChannel = channel
        self.controller = controller
    }
    
    func addService(_ service: FLTService) {
        services[service.serviceName()] = service
    }
    
    func getService(_ name: String) -> FLTService? {
        return services[name]
    }
    
    func setController(_ controller: UIViewController?){
        self.controller = controller
    }
    
    func setMethodChannel(_ safeMethodChannel: SafeMethodChannel?) {
        self.safeMethodChannel = safeMethodChannel
    }

    func getMethodChannel() -> SafeMethodChannel? {
        return safeMethodChannel
    }
    
    func onMethodCall(_ method: String, _ arguments: [String : Any], resultCallback: ResultCallback) {
        print("========================== \(method) ==========================")
        dump(arguments)
        print("========================== \(method) ==========================")
        if let serviceName = arguments[kFLTNimCoreService] as? String {
            if let service = getService(serviceName) {
                service.onMethodCalled(method, arguments, resultCallback)
            }
        }else {
            resultCallback.notImplemented()
        }
    }
    
}
