/*
 * Copyright (c) 2021 NetEase, Inc.  All rights reserved.
 * Use of this source code is governed by a MIT license that can be
 * found in the LICENSE file.
 */

import Foundation
import NIMSDK

enum TeamType: String {
    case GroupCreate = "createTeam"
    
    //获取群组
    case LocalAll = "queryTeamList" //本地所有群组
    case LocalSpecial = "queryTeam" //本地指定群组
    case RemoteSpecial = "searchTeam" //云端指定群组
    
    case DismissGroup = "dismissTeam" // 解散群组
    
    case SearchTeamIdByName = "searchTeamIdByName"
    case SearchTeamsByKeyword = "searchTeamsByKeyword"
    
    //入群操作
    case ApplyToTeam = "applyJoinTeam" //申请加入群组
    case PassApplyToTeam = "passApply" //申请入群通过
    case AddUser = "addMembersEx" //邀请入群
    case AcceptInvite = "acceptInvite"  //同意群邀请(仅限高级群)
    case RejectInvite = "rejectInvite"  //拒绝群邀请(仅限高级群)
    case FetchInviter = "getMemberInvitor"  //查询群成员入群邀请人

    case KickUsers = "removeMembers"  //踢人出群
    case QuitTeam = "quitTeam"  //主动退群
    case FetchTeamMembers = "queryMemberList" //获取群成员
    case TeamMember = "queryTeamMember" //获取指定群组成员
    case UpdateUserNick = "updateMemberNick" //修改群成员资料
    case UpdateMyCustomInfo = "updateMyCustomInfo" //修改群成员自定义属性
    case IsMyTeam = "isMyTeam" //判断自己是否在群里
    case TransferManager = "transferTeam"  //群主转让
    case AddManagers = "addManagers"  //添加管理员
    case RemoveManagers = "removeManagers" //移除管理员
    
    //禁言
    case MuteUser = "muteTeamMember" //禁言指定成员
    case MuteTeam = "muteAllTeamMember" //禁言全体普通成员
    case FetchTeamMutedMembers = "queryMutedTeamMembers" //查询禁言用户列表
    
    case UpdateTeamFields = "updateTeamFields"
    case UpdateTeam = "updateTeam"
    
    //群资料管理
    case UpdateTeamName = "updateTeamName"     //修改群名
    case UpdateTeamAvatar = "updateTeamAvatar" //修改群头像
    case UpdateTeamIntro = "updateTeamIntro"   //修改群介绍
    case UpdateTeamAnnouncement = "updateTeamAnnouncement" //修改群公告
    case UpdateTeamJoinMode = "updateTeamJoinMode" //修改群验证方式
    case UpdateTeamBeInviteMode = "updateTeamBeInviteMode" //修改被邀请人验证方式
    case UpdateTeamInviteMode = "updateTeamInviteMode"  //修改谁可以邀请其他人入群
    case UpdateTeamUpdateInfoMode = "updateTeamUpdateInfoMode" //修改谁可以修改群资料
    case UpdateTeamUpdateClientCustomMode = "updateTeamUpdateClientCustomMode" //修改谁可以修改群资料
    case UpdateTeamCustomInfo = "updateMyMemberExtension"  //修改群自定义属性
    
    case UpdateNotifyState = "muteTeam"   //群消息免打扰
    case NotifyStateForNewMsg = "notifyStateForNewMsg"  //查询群消息免打扰
}

class FLTTeamService: FLTBaseService, FLTService {
    
    override init() {
        super.init()
        NIMSDK.shared().teamManager.add(self)
    }
    
    deinit {
        NIMSDK.shared().teamManager.remove(self)
    }
    
    
    func serviceName() -> String {
        return ServiceType.TeamService.rawValue
    }
    
    func register(_ nimCore: NimCore) {
        self.nimCore = nimCore
        nimCore.addService(self)
    }
    
    func onMethodCalled(_ method: String, _ arguments: [String : Any], _ resultCallback: ResultCallback) {
        switch method {
        case TeamType.GroupCreate.rawValue:
            createGroup(arguments, resultCallback)
            break
            
        //获取群组
        case TeamType.LocalAll.rawValue: getLocalALl(arguments, resultCallback)
        case TeamType.LocalSpecial.rawValue: getSpecialTeam(arguments, resultCallback)
        case TeamType.RemoteSpecial.rawValue: getTeamFromServer(arguments, resultCallback)
        
        case TeamType.DismissGroup.rawValue: dismissGroup(arguments, resultCallback)
            
        //case TeamType.SearchTeamWithOption.rawValue: searchTeamWithOption(arguments, resultCallback)
        case TeamType.SearchTeamsByKeyword.rawValue: searchTeamWithOption(arguments, resultCallback)
        case TeamType.SearchTeamIdByName.rawValue: searchTeamWithOption(arguments, resultCallback)
        
        //入群操作
        case TeamType.ApplyToTeam.rawValue: applyToTeam(arguments, resultCallback)
        case TeamType.PassApplyToTeam.rawValue: passApplyToTeam(arguments, resultCallback)
        case TeamType.AddUser.rawValue: addUsers(arguments, resultCallback)
        case TeamType.AcceptInvite.rawValue: acceptInviteWithTeam(arguments, resultCallback)
        case TeamType.RejectInvite.rawValue: rejectInviteWithTeam(arguments, resultCallback)
        case TeamType.FetchInviter.rawValue: fetchInviterAccids(arguments, resultCallback)

        case TeamType.KickUsers.rawValue: kickUsers(arguments, resultCallback)
        case TeamType.QuitTeam.rawValue: quitTeam(arguments, resultCallback)
        case TeamType.FetchTeamMembers.rawValue: fetchTeamMembersFromServer(arguments, resultCallback)
        case TeamType.TeamMember.rawValue: teamMember(arguments, resultCallback)
        case TeamType.UpdateUserNick.rawValue: updateUserNick(arguments, resultCallback)
        case TeamType.UpdateMyCustomInfo.rawValue: updateMyCustomInfo(arguments, resultCallback)
        case TeamType.IsMyTeam.rawValue: isMyTeam(arguments, resultCallback)
        case TeamType.TransferManager.rawValue: transferManagerWithTeam(arguments, resultCallback)
        case TeamType.AddManagers.rawValue: addManagersToTeam(arguments, resultCallback)
        case TeamType.RemoveManagers.rawValue: removeManagersFromTeam(arguments, resultCallback)
        
        //禁言
        case TeamType.MuteUser.rawValue: updateMuteState(arguments, resultCallback)
        case TeamType.MuteTeam.rawValue: updateMuteAllState(arguments, resultCallback)
        case TeamType.FetchTeamMutedMembers.rawValue: fetchTeamMutedMembers(arguments, resultCallback)
            
        case TeamType.UpdateTeamFields.rawValue: updateTeamFields(arguments, resultCallback)
        case TeamType.UpdateTeam.rawValue: break
        
        //群资料管理
        case TeamType.UpdateTeamName.rawValue: updateTeamName(arguments, resultCallback)
        case TeamType.UpdateTeamAvatar.rawValue: updateTeamAvatar(arguments, resultCallback)
        case TeamType.UpdateTeamIntro.rawValue: updateTeamIntro(arguments, resultCallback)
        case TeamType.UpdateTeamAnnouncement.rawValue: updateTeamAnnouncement(arguments, resultCallback)
        case TeamType.UpdateTeamJoinMode.rawValue: updateTeamJoinMode(arguments, resultCallback)
        case TeamType.UpdateTeamBeInviteMode.rawValue: updateTeamBeInviteMode(arguments, resultCallback)
        case TeamType.UpdateTeamInviteMode.rawValue: updateTeamInviteMode(arguments, resultCallback)
        case TeamType.UpdateTeamUpdateInfoMode.rawValue: updateTeamUpdateInfoMode(arguments, resultCallback)
        case TeamType.UpdateTeamUpdateClientCustomMode.rawValue: updateTeamUpdateClientCustomMode(arguments, resultCallback)
        case TeamType.UpdateTeamCustomInfo.rawValue: updateTeamCustomInfo(arguments, resultCallback)
        
        case TeamType.UpdateNotifyState.rawValue: updateNotifyState(arguments, resultCallback)
        case TeamType.NotifyStateForNewMsg.rawValue: notifyStateForNewMsg(arguments, resultCallback)
        default:
            resultCallback.notImplemented()
            break
        }
    }
    
    private func createGroup(_ arguments: [String : Any], _ resultCallback: ResultCallback){
        if let option = arguments["createTeamOptions"] as? [String : Any],
           let createOption = NIMCreateTeamOption.fromDic(option) as? NIMCreateTeamOption,
           let users = arguments["members"] as? [String] {
            weak var weakSelf = self
            NIMSDK.shared().teamManager.createTeam(createOption, users: users) { error, teamid, failedUserIds in
                weakSelf?.teamCallback(error, failedUserIds, resultCallback)
            }
        }
    }
    
    private func getLocalALl(_ arguments: [String : Any], _ resultCallback: ResultCallback){
        let teams =  NIMSDK.shared().teamManager.allMyTeams()
        let ret = teams?.map({ team in team.toDic() })
        teamCallback(nil, ["teamList" : ret], resultCallback)
    }
    
    private func getSpecialTeam(_ arguments: [String : Any], _ resultCallback: ResultCallback){
        guard let teamId = getTeamId(arguments) else {
            errorCallBack(resultCallback, "parameter is error")
            return
        }
        let team = NIMSDK.shared().teamManager.team(byId: teamId)
        teamCallback(nil, team?.toDic(), resultCallback)
    }
    
    //从服务器获取群组
    private func getTeamFromServer(_ arguments: [String : Any], _ resultCallback: ResultCallback) {
        guard let teamId = getTeamId(arguments) else {
            errorCallBack(resultCallback, "parameter is error")
            return
        }
        weak var weakSelf = self
        NIMSDK.shared().teamManager.fetchTeamInfo(teamId, completion: { error, team in
            weakSelf?.teamCallback(error, team?.toDic(), resultCallback)
        });
    }
    
    //解散群
    private func dismissGroup(_ arguments: [String : Any], _ resultCallback: ResultCallback){
        guard let teamId = getTeamId(arguments) else {
            errorCallBack(resultCallback, "parameter is error")
            return
        }
        weak var weakSelf = self
        NIMSDK.shared().teamManager.dismissTeam(teamId) { error in
            weakSelf?.teamCallback(error, nil, resultCallback)
        }
    }
    
    //申请加入群组
    private func applyToTeam(_ arguments: [String : Any], _ resultCallback: ResultCallback){
        guard let teamId = getTeamId(arguments) else {
            errorCallBack(resultCallback, "parameter is error")
            return
        }
        let message = arguments["postscript"] as? String ?? ""
        weak var weakSelf = self
        NIMSDK.shared().teamManager.apply(toTeam: teamId, message: message) { error, status in
            //TODO: dart 返回 team 对象
            weakSelf?.teamCallback(error, ["status" : status], resultCallback)
        }
    }
    
    // 通过申请(仅限高级群)
    private func passApplyToTeam(_ arguments: [String : Any], _ resultCallback: ResultCallback){
        guard let teamId = getTeamId(arguments),
              let userId = getUserId(arguments) else {
            errorCallBack(resultCallback, "parameter is error")
            return
        }
        
        weak var weakSelf = self
        NIMSDK.shared().teamManager.passApply(toTeam: teamId, userId: userId) { error, status in
            weakSelf?.teamCallback(error, ["status" : status], resultCallback)
        }
    }
    
    //拒绝申请(仅限高级群)
    private func rejectApplyToTeam(_ arguments: [String : Any], _ resultCallback: ResultCallback){
        guard let teamId = getTeamId(arguments), let userId = getUserId(arguments), let reason = arguments["reason"] as? String else {
            errorCallBack(resultCallback, "parameter is error")
            return
        }
        
        weak var weakSelf = self
        NIMSDK.shared().teamManager.rejectApply(toTeam: teamId, userId: userId, rejectReason: reason) { error in
            weakSelf?.teamCallback(error, nil, resultCallback)
        }
    }
    
    //邀请用户入群
    private func addUsers(_ arguments: [String : Any], _ resultCallback: ResultCallback){
        guard let teamId = getTeamId(arguments), let users = arguments["users"] as? [String] else {
            errorCallBack(resultCallback, "parameter is error")
            return
        }
        
        weak var weakSelf = self
        let postscript = arguments["msg"] as? String
        let attach = arguments["customInfo"] as? String
        
        NIMSDK.shared().teamManager.addUsers(users, toTeam: teamId, postscript: postscript, attach: attach) { error, members in
            
            let array = NSArray()
            members?.forEach({ member in
                if let json = member.toDic(), let jsonString = weakSelf?.getJsonStringFromDictionary(json) {
                    array.adding(jsonString)
                }
            })
            if let ns_error = error as NSError? {
                weakSelf?.errorCallBack(resultCallback, ns_error.description, ns_error.code)
            }else {
                resultCallback.result(["code":0, "data": ["teamMemberExList" : array.yx_modelToJSONString()] as Any])
            }
        }
    }
    
    //同意群邀请(仅限高级群)
    private func acceptInviteWithTeam(_ arguments: [String : Any], _ resultCallback: ResultCallback) {
        guard let teamId = getTeamId(arguments),
              let invitorId = arguments["inviter"] as? String else {
            errorCallBack(resultCallback, "parameter is error")
            return
        }
        
        weak var weakSelf = self
        NIMSDK.shared().teamManager.acceptInvite(withTeam: teamId, invitorId: invitorId) { error in
            weakSelf?.teamCallback(error, nil, resultCallback)
        }
    }
    
    // 拒绝群邀请(仅限高级群)
    private func rejectInviteWithTeam(_ arguments: [String : Any], _ resultCallback: ResultCallback){
        guard let teamId = getTeamId(arguments),
              let invitorId = arguments["invitorId"] as? String ,
              let reason = arguments["reason"] as? String  else {
            errorCallBack(resultCallback, "parameter is error")
            return
        }
        
        weak var weakSelf = self
        NIMSDK.shared().teamManager.rejectInvite(withTeam: teamId, invitorId: invitorId, rejectReason: reason) { error in
            weakSelf?.teamCallback(error, nil, resultCallback)
        }
    }
    
    //查询群成员入群邀请人
    private func fetchInviterAccids(_ arguments: [String : Any], _ resultCallback: ResultCallback){
        guard let teamId = getTeamId(arguments),
              let memberIds = arguments["accids"] as? [String] else {
            errorCallBack(resultCallback, "parameter is error")
            return
        }
        
        weak var weakSelf = self
        NIMSDK.shared().teamManager.fetchInviterAccids(teamId, withTargetMembers: memberIds) { error, inviters in
            weakSelf?.teamCallback(error, inviters, resultCallback)
        }
    }
    
    //踢人出群
    private func kickUsers(_ arguments: [String : Any], _ resultCallback: ResultCallback){
        guard let teamId = getTeamId(arguments),
              let userIds = arguments["members"] as? [String] else {
            errorCallBack(resultCallback, "parameter is error")
            return
        }
        
        weak var weakSelf = self
        NIMSDK.shared().teamManager.kickUsers(userIds, fromTeam: teamId) { error  in
            weakSelf?.teamCallback(error, nil, resultCallback)
        }
    }
    
    //主动退群
    private func quitTeam(_ arguments: [String : Any], _ resultCallback: ResultCallback){
        guard let teamId = getTeamId(arguments) else {
            errorCallBack(resultCallback, "parameter is error")
            return
        }
        
        weak var weakSelf = self
        NIMSDK.shared().teamManager.quitTeam(teamId) { error in
            weakSelf?.teamCallback(error, nil, resultCallback)
        }
    }
    
    //获取群组成员列表
    private func fetchTeamMembersFromServer(_ arguments: [String : Any], _ resultCallback: ResultCallback) {
        guard let teamId = getTeamId(arguments) else {
            errorCallBack(resultCallback, "parameter is error")
            return
        }
        
        weak var weakSelf = self
        NIMSDK.shared().teamManager.fetchTeamMembers(teamId) { error, members in
            let ret = members?.map({ member in
                member.toDic()
            })
            weakSelf?.teamCallback(error, ["teamMemberList" : ret], resultCallback)
        }
    }
    
    //获取指定群组成员
    private func teamMember(_ arguments: [String : Any], _ resultCallback: ResultCallback){
        guard let teamId = getTeamId(arguments),
              let userId = getUserId(arguments) else {
            errorCallBack(resultCallback, "parameter is error")
            return
        }
        
        let member = NIMSDK.shared().teamManager.teamMember(userId, inTeam: teamId)
        teamCallback(nil, member?.toDic, resultCallback)
    }
    
    //修改群成员昵称
    private func updateUserNick(_ arguments: [String : Any], _ resultCallback: ResultCallback){
        guard let teamId = getTeamId(arguments),
              let userId = getUserId(arguments),
              let newNick = arguments["nick"] as? String else {
            errorCallBack(resultCallback, "parameter is error")
            return
        }
        
        weak var weakSelf = self
        NIMSDK.shared().teamManager.updateUserNick(userId, newNick: newNick, inTeam: teamId) { error in
            weakSelf?.teamCallback(error, nil, resultCallback)
        }
    }
    
    //修改群成员自定义属性
    private func updateMyCustomInfo(_ arguments: [String : Any], _ resultCallback: ResultCallback){
        guard let teamId = getTeamId(arguments), let newInfo = arguments["newInfo"] as? String else {
            errorCallBack(resultCallback, "parameter is error")
            return
        }
        
        weak var weakSelf = self
        NIMSDK.shared().teamManager.updateMyCustomInfo(newInfo, inTeam: teamId) { error in
            weakSelf?.teamCallback(error, nil, resultCallback)
        }
    }
    
    //判断自己是否在群里
    private func isMyTeam(_ arguments: [String : Any], _ resultCallback: ResultCallback){
        guard let teamId = getTeamId(arguments) else {
            errorCallBack(resultCallback, "parameter is error")
            return
        }
        let isMyTeam = NIMSDK.shared().teamManager.isMyTeam(teamId)
        teamCallback(nil, ["isMyteam": isMyTeam], resultCallback)
    }
    
    //群主转让
    private func transferManagerWithTeam(_ arguments: [String : Any], _ resultCallback: ResultCallback){
        guard let teamId = getTeamId(arguments),
              let newOwnerId = arguments["account"] as? String,
              let isLeave = arguments["quit"] as? Bool else {
            errorCallBack(resultCallback, "parameter is error")
            return
        }
        
        weak var weakSelf = self
        NIMSDK.shared().teamManager.transferManager(withTeam: teamId, newOwnerId: newOwnerId, isLeave: isLeave) { error  in
            weakSelf?.teamCallback(error, nil, resultCallback)
        }
    }
    
    //添加管理员
    private func addManagersToTeam(_ arguments: [String : Any], _ resultCallback: ResultCallback){
        guard let teamId = getTeamId(arguments),
              let userIds = arguments["managers"] as? [String] else {
            errorCallBack(resultCallback, "parameter is error")
            return
        }
        
        weak var weakSelf = self
        NIMSDK.shared().teamManager.addManagers(toTeam: teamId, users: userIds) { error in
            weakSelf?.teamCallback(error, nil, resultCallback)
        }
    }
    
    //移除管理员
    private func removeManagersFromTeam(_ arguments: [String : Any], _ resultCallback: ResultCallback){
        guard let teamId = getTeamId(arguments),
              let userIds = arguments["managers"] as? [String] else {
            errorCallBack(resultCallback, "parameter is error")
            return
        }
        
        weak var weakSelf = self
        NIMSDK.shared().teamManager.removeManagers(fromTeam: teamId, users: userIds) { error  in
            weakSelf?.teamCallback(error, nil, resultCallback)
        }
    }
    
    //禁言指定成员
    private func updateMuteState(_ arguments: [String : Any], _ resultCallback: ResultCallback){
        guard let teamId = getTeamId(arguments),
              let userId = getUserId(arguments),
              let mute = arguments["mute"] as? Bool else {
            errorCallBack(resultCallback, "parameter is error")
            return
        }
        
        weak var weakSelf = self
        NIMSDK.shared().teamManager.updateMuteState(mute, userId:userId , inTeam: teamId) { error in
            weakSelf?.teamCallback(error, nil, resultCallback)
        }
    }
    
    //禁言全体普通成员
    private func updateMuteAllState(_ arguments: [String : Any], _ resultCallback: ResultCallback){
        guard let teamId = getTeamId(arguments),
              let mute = arguments["mute"] as? Bool else {
            errorCallBack(resultCallback, "parameter is error")
            return
        }
        
        weak var weakSelf = self
        NIMSDK.shared().teamManager.updateMuteState(mute, inTeam: teamId) { error in
            weakSelf?.teamCallback(error, nil, resultCallback)
        }
    }
    
    //查询禁言用户列表
    private func fetchTeamMutedMembers(_ arguments: [String : Any], _ resultCallback: ResultCallback){
        guard let teamId = getTeamId(arguments) else {
            errorCallBack(resultCallback, "parameter is error")
            return
        }
        
        weak var weakSelf = self
        NIMSDK.shared().teamManager.fetchTeamMutedMembers(teamId) { error, members in
            let ret = members?.map({ member in
                member.toDic()
            })
            weakSelf?.teamCallback(error, ret, resultCallback)
        }
    }
    
    private func updateTeamFields(_ arguments: [String : Any], _ resultCallback: ResultCallback){
        guard let teamId = getTeamId(arguments),
              let fields = arguments["fields"] as? [String : String] else {
            errorCallBack(resultCallback, "parameter is error")
            return
        }
        weak var weakSelf = self
        var convert = [NSNumber: String]()
        fields.forEach { (key: String, value: String) in
            if let type = FLT_NIMTeamUpdateTag(rawValue: key)?.convertToNIMTeamUpdateTag()?.rawValue{
                convert[NSNumber(integerLiteral: type)] = value
            }
        }
        NIMSDK.shared().teamManager.updateTeamInfos(convert, teamId: teamId) { error in
            weakSelf?.teamCallback(error, nil, resultCallback)
        }
    }
    
    private func updateTeam(_ arguments: [String : Any], _ resultCallback: ResultCallback){
        guard let teamId = getTeamId(arguments),
              let field = arguments["field"] as? String,
              let value = arguments["value"],
              let fieldType = FLT_NIMTeamUpdateTag(rawValue: field) else {
            errorCallBack(resultCallback, "parameter is error")
            return
        }
        
        var arg = [String : Any]()
        arg["teamId"] = teamId

        switch fieldType {

        case .announcement:
            arg["announcement"] = value
            updateTeamAnnouncement(arg, resultCallback)
        case .beInviteMode:
            arg["beInviteMode"] = value
            updateTeamBeInviteMode(arg, resultCallback)
        case .extensiontype:
            arg["clientCustomMode"] = value
            updateTeamUpdateClientCustomMode(arg, resultCallback)
        case .icon:
            arg["teamAvatarUrl"] = value
            updateTeamAvatar(arg, resultCallback)
        case .introduce:
            arg["intro"] = value
            updateTeamIntro(arg, resultCallback)
        case .inviteMode:
            arg["inviteMode"] = value
            updateTeamInviteMode(arg, resultCallback)
        case .name:
            arg["teamName"] = value
            updateTeamName(arg, resultCallback)

        case .teamExtensionUpdateMode:
            arg["extension"] = value
            updateTeamCustomInfo(arg, resultCallback)
        case .teamUpdateMode:
            arg["updateInfoMode"] = value
            updateTeamUpdateInfoMode(arg, resultCallback)
        case .verifyType:
            arg["joinMode"] = value
            updateTeamJoinMode(arg, resultCallback)
        default:
            resultCallback.notImplemented()
            break
        }
    }
    
    //修改群昵称
    private func updateTeamName(_ arguments: [String : Any], _ resultCallback: ResultCallback) {
        guard let teamId = getTeamId(arguments),
              let teamName = arguments["teamName"] as? String else {
            errorCallBack(resultCallback, "parameter is error")
            return
        }
        weak var weakSelf = self
        NIMSDK.shared().teamManager.updateTeamName(teamName, teamId: teamId) { error in
            weakSelf?.teamCallback(error, nil, resultCallback)
        }
    }
    
    //修改群头像
    private func updateTeamAvatar(_ arguments: [String : Any], _ resultCallback: ResultCallback) {
        guard let teamId = getTeamId(arguments),
              let teamAvatarUrl = arguments["teamAvatarUrl"] as? String else {
            errorCallBack(resultCallback, "parameter is error")
            return
        }
        weak var weakSelf = self
        NIMSDK.shared().teamManager.updateTeamAvatar(teamAvatarUrl, teamId: teamId) { error in
            weakSelf?.teamCallback(error, nil, resultCallback)
        }
    }
    
    //修改群介绍
    private func updateTeamIntro(_ arguments: [String : Any], _ resultCallback: ResultCallback) {
        guard let teamId = getTeamId(arguments),
              let intro = arguments["intro"] as? String else {
            errorCallBack(resultCallback, "parameter is error")
            return
        }
        weak var weakSelf = self
        NIMSDK.shared().teamManager.updateTeamIntro(intro, teamId: teamId) { error in
            weakSelf?.teamCallback(error, nil, resultCallback)
        }
    }
    
    //修改群公告
    private func updateTeamAnnouncement(_ arguments: [String : Any], _ resultCallback: ResultCallback) {
        guard let teamId = getTeamId(arguments),
              let announcement = arguments["announcement"] as? String else {
            errorCallBack(resultCallback, "parameter is error")
            return
        }
        weak var weakSelf = self
        NIMSDK.shared().teamManager.updateTeamIntro(announcement, teamId: teamId) { error in
            weakSelf?.teamCallback(error, nil, resultCallback)
        }
    }
    
    //修改群验证方式
    private func updateTeamJoinMode(_ arguments: [String : Any], _ resultCallback: ResultCallback) {
        guard let teamId = getTeamId(arguments),
              let joinMode = arguments["joinMode"] as? Int,
              let mode = NIMTeamJoinMode(rawValue: joinMode) else {
            errorCallBack(resultCallback, "parameter is error")
            return
        }
        weak var weakSelf = self
        NIMSDK.shared().teamManager.update(mode, teamId: teamId) { error in
            weakSelf?.teamCallback(error, nil, resultCallback)
        }
    }
    
    //修改被邀请人验证方式
    private func updateTeamBeInviteMode(_ arguments: [String : Any], _ resultCallback: ResultCallback) {
        guard let teamId = getTeamId(arguments),
              let beInviteMode = arguments["beInviteMode"] as? Int,
              let mode = NIMTeamBeInviteMode(rawValue: beInviteMode) else {
            errorCallBack(resultCallback, "parameter is error")
            return
        }
        weak var weakSelf = self
        NIMSDK.shared().teamManager.update(mode, teamId: teamId, completion: { error in
            weakSelf?.teamCallback(error, nil, resultCallback)
        })
    }
    
    //修改谁可以邀请其他人入群
    private func updateTeamInviteMode(_ arguments: [String : Any], _ resultCallback: ResultCallback) {
        guard let teamId = getTeamId(arguments),
              let inviteMode = arguments["inviteMode"] as? Int,
              let mode = NIMTeamInviteMode(rawValue: inviteMode) else {
            errorCallBack(resultCallback, "parameter is error")
            return
        }
        weak var weakSelf = self
        NIMSDK.shared().teamManager.update(mode, teamId: teamId, completion: { error in
            weakSelf?.teamCallback(error, nil, resultCallback)
        })
    }
    
    //修改谁可以修改群资料
    private func updateTeamUpdateInfoMode(_ arguments: [String : Any], _ resultCallback: ResultCallback) {
        guard let teamId = getTeamId(arguments),
              let updateInfoMode = arguments["updateInfoMode"] as? Int,
              let mode = NIMTeamUpdateInfoMode(rawValue: updateInfoMode) else {
            errorCallBack(resultCallback, "parameter is error")
            return
        }
        weak var weakSelf = self
        NIMSDK.shared().teamManager.update(mode, teamId: teamId) { error in
            weakSelf?.teamCallback(error, nil, resultCallback)
        }
    }
    
    //修改谁可以修改群自定义属性
    private func updateTeamUpdateClientCustomMode(_ arguments: [String : Any], _ resultCallback: ResultCallback) {
        guard let teamId = getTeamId(arguments),
              let clientCustomMode = arguments["clientCustomMode"] as? Int,
              let mode = NIMTeamUpdateClientCustomMode(rawValue: clientCustomMode) else {
            errorCallBack(resultCallback, "parameter is error")
            return
        }
        weak var weakSelf = self
        NIMSDK.shared().teamManager.update(mode, teamId: teamId) { error in
            weakSelf?.teamCallback(error, nil, resultCallback)
        }
    }
    
    //修改群自定义属性
    private func updateTeamCustomInfo(_ arguments: [String : Any], _ resultCallback: ResultCallback) {
        guard let teamId = getTeamId(arguments),
              let info = arguments["extension"] as? String else {
            errorCallBack(resultCallback, "parameter is error")
            return
        }
        
        weak var weakSelf = self
        NIMSDK.shared().teamManager.updateTeamCustomInfo(info, teamId: teamId) { error in
            weakSelf?.teamCallback(error, nil, resultCallback)
        }
    }
    
    // 消息免打扰
    private func updateNotifyState(_ arguments: [String : Any], _ resultCallback: ResultCallback){
        guard let teamId = getTeamId(arguments),
              let notifyState = arguments["notifyType"] as? Int,
              let state = NIMTeamNotifyState(rawValue: notifyState) else {
            errorCallBack(resultCallback, "parameter is error")
            return
        }
        
        weak var weakSelf = self
        NIMSDK.shared().teamManager.update(state, inTeam: teamId) { error in
            weakSelf?.teamCallback(error, nil, resultCallback)
        }
    }
    
    //消息免打扰查询
    private func notifyStateForNewMsg(_ arguments: [String : Any], _ resultCallback: ResultCallback){
        guard let teamId = getTeamId(arguments) else {
            errorCallBack(resultCallback, "parameter is error")
            return
        }
        let state = NIMSDK.shared().teamManager.notifyState(forNewMsg: teamId)
        teamCallback(nil, ["state" : state], resultCallback)
    }
    
    //群组检索
    private func searchTeamWithOption(_ arguments: [String : Any], _ resultCallback: ResultCallback){

        if let name = arguments["name"] as? String  {
            let option = NIMTeamSearchOption()
            option.searchContent = name
            option.searchContentOption = .optiontName
            
            weak var weakSelf = self
            NIMSDK.shared().teamManager.searchTeam(with: option) { error , teams in
                let ret = teams?.map({ team in team.teamId })
                weakSelf?.teamCallback(error, ["teamNameList" : ret], resultCallback)
            }
            
            return
        }
        
        if let keyword = arguments["keyword"] as? String {
            let option = NIMTeamSearchOption()
            option.searchContent = keyword
            option.searchContentOption = .optionTeamAll
            
            weak var weakSelf = self
            NIMSDK.shared().teamManager.searchTeam(with: option) { error , teams in
                let ret = teams?.map({ team in
                    team.toDic()
                })
                weakSelf?.teamCallback(error, ["teamList" : ret], resultCallback)
            }
            return
        }
        errorCallBack(resultCallback, "parameter is error")
    }
    
    private func teamCallback(_ error: Error?, _ data: Any?, _ resultCallback: ResultCallback){
        if let ns_error = error as NSError? {
            errorCallBack(resultCallback, ns_error.description, ns_error.code)
        }else {
            successCallBack(resultCallback, data)
        }
    }
}

extension FLTTeamService: NIMTeamManagerDelegate {
    
    func onTeamMemberChanged(_ team: NIMTeam) {
        
    }
    
    func onTeamRemoved(_ team: NIMTeam) {
        notifyEvent(serviceName(), "onTeamListRemove", ["teamList" : [team.toDic() as Any]])
    }
    
    func onTeamUpdated(_ team: NIMTeam) {
        notifyEvent(serviceName(), "onTeamListUpdate", ["team" : [team.toDic() as Any]])
    }
}
