// Copyright (c) 2021 NetEase, Inc.  All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

part of nim_core;

class SettingsService {
  factory SettingsService() {
    if (_singleton == null) {
      _singleton = SettingsService._();
    }
    return _singleton!;
  }

  SettingsService._() {
    SettingsServicePlatform.instance = _SettingsServiceMobile();
  }

  static SettingsService? _singleton;

  SettingsServicePlatform get _platform => SettingsServicePlatform.instance;

  ///
  /// 设置桌面端(PC/WEB)在线时，移动端是否需要推送
  ///
  /// [enable] true 桌面端在线时移动端需推送；false 桌面端在线时移动端不需推送
  Future<NIMResult<void>> enableMobilePushWhenPCOnline({
    required bool enable,
  }) {
    return _platform.enableMobilePushWhenPCOnline(enable);
  }

  ///
  /// 查询桌面端(PC/WEB)在线时，移动端推送开关状态
  ///
  Future<NIMResult<bool>> isMobilePushEnabledWhenPCOnline() {
    return _platform.isMobilePushEnabledWhenPCOnline();
  }

  /// 配置消息提醒，仅支持 Android 平台，iOS平台请关闭通知权限
  ///
  /// <p>[enableRegularNotification] - 普通消息提醒开关
  /// <p>[enableRevokeMessageNotification] - 消息撤回是否提醒
  Future<NIMResult<void>> enableNotification({
    required bool enableRegularNotification,
    required bool enableRevokeMessageNotification,
  }) {
    return _platform.enableNotification(
      enableRegularNotification: enableRegularNotification,
      enableRevokeMessageNotification: enableRevokeMessageNotification,
    );
  }

  /// 更新通知栏设置，仅支持Android平台
  Future<NIMResult<void>> updateNotificationConfig(
      NIMStatusBarNotificationConfig config) async {
    return _platform.updateNotificationConfig(config);
  }

  /// 开启或关闭消息推送；当设置为 false 时，该客户端将接收不到来自云信体系内的推送。
  Future<NIMResult<void>> enablePushService(bool enable) {
    return _platform.enablePushService(enable);
  }

  /// 查询当前推送服务开关
  Future<NIMResult<bool>> isPushServiceEnabled() {
    return _platform.isPushServiceEnabled();
  }

  /// 获取当前免打扰设置
  Future<NIMResult<NIMPushNoDisturbConfig>> getPushNoDisturbConfig() {
    return _platform.getPushNoDisturbConfig();
  }

  /// 设置免打扰配置
  Future<NIMResult<void>> setPushNoDisturbConfig(
      NIMPushNoDisturbConfig config) async {
    return _platform.setPushNoDisturbConfig(config);
  }

  /// 查询推送是否显示详情
  Future<NIMResult<bool>> isPushShowDetailEnabled() {
    return _platform.isPushShowDetailEnabled();
  }

  /// 设置推送是否显示详情
  Future<NIMResult<void>> enablePushShowDetail(bool enable) {
    return _platform.enablePushShowDetail(enable);
  }

  /// 更新iOS deviceToken
  Future<NIMResult<void>> updateAPNSToken(String token) async{
    return _platform.updateAPNSToken(token);
  }
}
