// Copyright (c) 2021 NetEase, Inc.  All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.
part of nim_core;

class _SettingsServiceMobile extends SettingsServicePlatform {
  @override
  Future onEvent(String method, arguments) {
    throw UnimplementedError();
  }

  @override
  String get serviceName => 'SettingsService';

  @override
  Future<NIMResult<void>> enableMobilePushWhenPCOnline(bool enable) async {
    return NIMResult.fromMap(
      await invokeMethod(
        'enableMobilePushWhenPCOnline',
        arguments: {
          'enable': enable,
        },
      ),
    );
  }

  @override
  Future<NIMResult<bool>> isMobilePushEnabledWhenPCOnline() async {
    return NIMResult.fromMap(
      await invokeMethod(
        'isMobilePushEnabledWhenPCOnline',
      ),
    );
  }

  @override
  Future<NIMResult<void>> enableNotification({
    required bool enableRegularNotification,
    required bool enableRevokeMessageNotification,
  }) async {
    if (Platform.isAndroid) {
      return NIMResult.fromMap(
        await invokeMethod(
          'enableNotification',
          arguments: {
            'enableRegularNotification': enableRegularNotification,
            'enableRevokeMessageNotification': enableRevokeMessageNotification,
          },
        ),
      );
    }
    return NIMResult(-1, null, 'Support Android platform only');
  }

  Future<NIMResult<void>> updateNotificationConfig(
      NIMStatusBarNotificationConfig config) async {
    if (Platform.isAndroid) {
      return NIMResult.fromMap(
        await invokeMethod(
          'updateNotificationConfig',
          arguments: config.toMap(),
        ),
      );
    }
    return NIMResult(-1, null, 'Support Android platform only');
  }

  Future<NIMResult<void>> enablePushService(bool enable) async {
    return NIMResult.fromMap(
      await invokeMethod(
        'enablePushService',
        arguments: {
          'enable': enable,
        },
      ),
    );
  }

  Future<NIMResult<bool>> isPushServiceEnabled() async {
    return NIMResult.fromMap(
      await invokeMethod(
        'isPushServiceEnabled',
      ),
    );
  }

  Future<NIMResult<NIMPushNoDisturbConfig>> getPushNoDisturbConfig() async {
    return NIMResult.fromMap(
      await invokeMethod(
        'getPushNoDisturbConfig',
      ),
      convert: (map) {
        return NIMPushNoDisturbConfig.fromMap(Map<String, dynamic>.from(map));
      },
    );
  }

  Future<NIMResult<void>> setPushNoDisturbConfig(
      NIMPushNoDisturbConfig config) async {
    return NIMResult.fromMap(
      await invokeMethod(
        'setPushNoDisturbConfig',
        arguments: config.toMap(),
      ),
    );
  }

  Future<NIMResult<bool>> isPushShowDetailEnabled() async {
    return NIMResult.fromMap(
      await invokeMethod(
        'isPushShowDetailEnabled',
      ),
    );
  }

  Future<NIMResult<void>> enablePushShowDetail(bool enable) async {
    return NIMResult.fromMap(
      await invokeMethod(
        'enablePushShowDetail',
        arguments: {
          'enable': enable,
        },
      ),
    );
  }

  /// 更新iOS deviceToken
  @override
  Future<NIMResult<void>> updateAPNSToken(String token) async{
    return NIMResult.fromMap(
      await invokeMethod('updateAPNSToken', arguments:{'token':token})
    );
  }
}

